import turbogears
from odict import odict
from turbogears import controllers, expose
from turbogears import flash, redirect, url

from paraxip.gateway_gui import extjs
from paraxip.gateway_gui_widgets.tabpstn  import TabPSTN
from paraxip.gateway_gui_widgets.treeviewpstn  import TreeViewPSTN

from netborderPyInfra import PyGlobalConfig as globalConfig
from pstnconfig import model
from operator import itemgetter
import simplejson
import os
import socket

import logging
log = logging.getLogger("pstnconfig.controllers")

class ConfigController(controllers.Controller):

  def __init__ (self, in_strProjectPath, in_webServiceObj):
    log.info("Initializing config controller")
    strSrcFile = os.path.join(in_strProjectPath, "pstnconfig/static/"
                                         "javascript/widgets_src/editor.js")
    self.isConfigOutOfSynch = False
    self.extTree = TreeViewPSTN('pstnTree', renderTo='tree', 
                                fetch='config/fetchTree', 
                                src_file=strSrcFile)
    
    #Elements that will build our tabber
    tabs = odict()
    #Each elements of the tabs dictionnary has the format:
    # tabs['Tab Text'] = ['/autoloadUrl','tabId']
    tabs['Routing Rules'] = ['','routingRulesTabId',"Edit the routing rules"]
    tabs['PSTN Config'] = ['','pstnEditorTabId',"Edit pstn configuration"]
    
    items = [{'title': k, 'autoLoad': v[0], 'id': v[1], 'tabTip': v[2]} for k, v in tabs.items()]
    tabList = [(k.replace(" ","").lower(),v[0],v[1],v[2]) for k, v in tabs.items()]
    tabs = dict(tabs)
    
    strSrcFile = os.path.join(in_strProjectPath, "pstnconfig/static/"
                              "javascript/widgets_src/pstn_config_tabs.js")
    
    self.extSubTabs = TabPSTN('configSubTabs',tabs=tabs, items=items, 
                              tabList=tabList, width= 757, height=595,
                              src_file=strSrcFile,children=[self.extTree])

    self.routingRulesPath = os.path.join(globalConfig.getParameter("netborder.Installation.Directory"),
                                         "config/routing-rules.xml")
    log.info("Routing rules path: %s" % self.routingRulesPath)
    self.webService = in_webServiceObj

  def getSubTabs (self):
    return self.extSubTabs

  @expose()
  def index (self):
    """Displays the configuration options to the user"""

    return dict(extSubTabs=extSubTabs)

  @expose(template='pstnconfig.templates.config.routing')
  def routing (self):
    """Displays the routing rules"""
    return {}

  @expose()
  def changeConfigOutOfSynch(self, newValue):
    log.info("Changing isConfigOutOfSynch to: %s" % newValue)
    if newValue == "true":
      self.isConfigOutOfSynch = True
    else: 
      self.isConfigOutOfSynch = False
    return {}

  @expose()
  def getIsConfigOutOfSynch(self, _dc=''):
    return {'isConfigOutOfSynch':self.isConfigOutOfSynch}

  @expose()
  def getRoutingRules(self):
    log.info("Getting routing rules")
    try:
      f = open(self.routingRulesPath,'r')
      # Read all content
      strFileContent = f.read()
      f.close()
    except IOError:
      log.error("Cannot open file ", self.routingRulesPath)               
      return "" 
    return {'routingRulesText':strFileContent}

  @expose()
  def saveRoutingRules(self, **data):
    log.info("Saving routing rules: %s" % data)
    routingText = simplejson.loads(data['xml'])['routingRulesEditField']

    result = self.validateRoutingRules(**data)
    result = simplejson.loads(result)

    if result['isValid'] == 'true':
      try:
         f = open(self.routingRulesPath,'w')
         f.write(routingText)
         f.close()
      except IOError:
        log.error("Cannot open file ", self.routingRulesPath)               
        return {'isValid':result['isValid'],'validationMsg':result['errorMsg'],'savingMsg':" Cannot open file %s" % self.routingRulesPath}
      return {'isValid':result['isValid'],'validationMsg':result['errorMsg'],'savingMsg':' File saved!'}
    else:
      return {'isValid':result['isValid'],'validationMsg':result['errorMsg'],'savingMsg':' File not saved!'}

  ##########################################################################
  # Validate the Routing rules
  # @param xml: the xml string representing the routing rules
  @expose()
  def validateRoutingRules(self, **xml):
    log.info("Routing rules string to validate: " % xml)
    rrXmlStr = simplejson.loads(xml['xml'])['routingRulesEditField']
    result = {}
    try:
      wsResult = self.webService.validateRoutingRules(rrXmlStr)
      result['m-bExecSuccess'] = wsResult['m-bExecSuccess']
      result['m-strStatusText'] = wsResult['m-strStatusText']

      if result['m-bExecSuccess'] == 'true':
        result['m-strStatusText'] = "XML file is valid."
      else:
        result['m-strStatusText'] = "XML file is not valid: " + result['m-strStatusText']
    except socket.error:
      result['m-bExecSuccess'] = 'true'
      result['m-strStatusText'] = 'Unable to validate file. Gateway must be started in order to proceed. ' + \
                                  'The file was saved but may be invalid.'
    return {'isValid':result['m-bExecSuccess'],'errorMsg':result['m-strStatusText']}
 
  @expose()
  def editor(self):
    return dict(extTree = self.extTree)

  ##########################################################################
  # Fetch Data provider for the root Ext.tree.AsyncTreeNode in the TreeView
  # Returns a json-encoded string a tree representation of the configuration
  @expose()
  def fetchTree(self, node):
    # Load current pstnconfig model if not done already
    # TODO check for errors
    if model.currentConfig is None and model.PSTNConfigFileExists():      
      model.currentConfig = model.parse(model.getPSTNConfigFileName())              
    return simplejson.dumps(model.getJSONForEditor(model.currentConfig))

