//------------------------------------------------------------
//  This file includes all the objects needed by the editor   |
//  to display the informations on the boards. Also includes  |
//  the method used to save the informations changed by the   |
//  user.                                                     |
//------------------------------------------------------------
Ext.namespace("Paraxip.config.boards");

//-----------------------------------------------
// Modify boards and interfaces methods
//-----------------------------------------------
var saveBoard = function(formButton){
  var form = formButton.ownerCt.getForm();
  var el = form.getEl();

  el.dom.action = 'boards/saveBoards';
  el.dom.method = 'POST';
  var boardInfo = formButton.ownerCt.getItemsValueAsJson();
  form.submit({
    params:{
      'boardInfo': boardInfo
    },
    success: function(){
       Paraxip.isConfigOutOfSynch(true);
       reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });

};

var saveInterface = function(formButton){
  var form = formButton.ownerCt.getForm();
  var el = form.getEl();

  el.dom.action = 'boards/saveInterfaces';
  el.dom.method = 'POST';

  // getItemsValueAsJson is a Ext.Container override
  var interfaceInfo = formButton.ownerCt.getItemsValueAsJson();
  form.submit({
    params:{
      'interfaceInfo': interfaceInfo
    },
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });

};

var boardCancelFunc = function(){
  reloadTree(true);
};

var formCancel = function (form){
  form.scope.hide();
};

//-----------------------------------------------
// Add/Delete Interfaces methods
//-----------------------------------------------
var addInterfaceHandler = function(){
  //used to get board id
  addInterfaceForm.panelOwner = this.ownerCt;
  addInterfaceWindow.show();
};

var deleteInterfaceHandler = function(){
  var currentGrid = this;
  var selectedRecords = currentGrid.getSelectionModel().getSelections();

  if (selectedRecords !== []) {
    var ifArray = new Array();
    for (var i = 0; i < selectedRecords.length; i++) {
      ifArray.push(selectedRecords[i].data);
    }
    ifArray = Ext.util.JSON.encode(ifArray);
    Ext.Ajax.request({
      url: 'boards/deleteInterfaceFromEditor',
      method: 'POST',
      disableCaching: false,
      params: {"selectedInterface":ifArray},
      success: function(){
        Paraxip.isConfigOutOfSynch(true);
        for (var index = 0; index < selectedRecords.length; index++) {
         currentGrid.store.remove(selectedRecords[index]);
        }
        reloadTree(true);
      },
      failure: Paraxip.failureMessage
    });
  }
};

var addInterface = function(){
  var boardId = this.ownerCt.panelOwner.items.items[0].items.items[1].getValue();
  //submit information
  var form = addInterfaceForm.getForm();
  var el = form.getEl();

  el.dom.action = 'boards/addInterfaceFromEditor';
  el.dom.method = 'POST';
  form.submit({
    params:{
      'interfaceName': addInterfaceForm.getComponent("interfaceName").getValue(),
      'boardID': boardId
    },
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });

  addInterfaceWindow.hide();
};

//-----------------------------------------------
// Add/Delete Boards methods
//-----------------------------------------------
var addBoardHandler = function(){
 addBoardWindow.show(gridBoards);
};

var deleteBoardHandler = function(){
  var selectedRecords = gridBoards.getSelectionModel().getSelections();

  if (selectedRecords !== []) {
    if ( gridBoards.store.getCount() - selectedRecords.length < 1 ){
      Ext.Msg.alert("Can not delete all boards!",
        "At least one board is necessary for the system to work properly. " +
        "Please add a new board before deleting the selected board(s)");
    } else {
      var boards = new Array();
      for (var i = 0; i < selectedRecords.length; i++) {
        boards.push(selectedRecords[i].data);
      }
      boards = Ext.util.JSON.encode(boards);
    //submit information
      Ext.Ajax.request({
        url: 'boards/deleteBoardFromEditor',
        method: 'POST',
        disableCaching: false,
        params: {"boards":boards},
        success: function(){
          Paraxip.isConfigOutOfSynch(true);
          reloadTree(true);
        },
        failure: Paraxip.failureMessage
      });
    }
  }
};

var addBoard = function(){
  //submit information
  var form = addBoardForm.getForm();
  var el = form.getEl();

  el.dom.action = 'boards/addBoardFromEditor';
  el.dom.method = 'POST';
  var newBoardData = addBoardForm.getItemsValueAsJson();
  form.submit({
    params:{
      'newBoardData'  : newBoardData
    },
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      reloadTree(true);
      Ext.Msg.alert("Board not assigned to a config!","The board you just added is"+
                   " not assigned to any config. Please manually assign board to a"+
                   " physical config and a call control config.");
    },
    failure: Paraxip.failureMessage
  });

  addBoardWindow.hide();
};

//-----------------------------------------------
// Intefaces Grids
//-----------------------------------------------
var gridInterfacesDigital = new Ext.grid.GridPanel({
  title       : 'Interfaces Grid',
  id          : 'gridInterfacesDigital',
  autoScroll  : true,
  layout      : 'fit',
  store       : new Ext.data.JsonStore({
    id: 'gridInterfacesDigital',
    url: 'boards/getGridDigitalInterfacesData',
    root: 'data',
    //,autoLoad: true
    fields: [
      {name: 'id'},
      {name: 'displayName'},
      {name: 'wanpipe'},
      {name: 'nodeId'} //Not displayed
    ]
  }),
  columns    :[
    {header: "ID", width: 35, sortable: true, dataIndex: 'id', css:'padding:5px;'},
    {header: "Name", width: 200, sortable: true, dataIndex: 'displayName', css:'padding:5px;'},
    {header: "Wanpipe", width: 100, sortable: true, dataIndex: 'wanpipe', css:'padding:5px;'}
  ],
  viewConfig :{forceFit:true}
}); // eo apply

//gridInterfacesDigital.addButton("Add Interface",addInterfaceHandler,gridInterfacesDigital);
//gridInterfacesDigital.addButton("Remove Interface(s)",deleteInterfaceHandler,gridInterfacesDigital);

gridInterfacesDigital.on('rowdblclick', function(grid, rowIndex){
  var record = grid.store.getAt(rowIndex);
  goToNode(record.get('nodeId'));
});

var gridInterfacesAnalog = new Ext.grid.GridPanel({
  title       : 'Interfaces Grid',
  id          : 'gridInterfacesAnalog',
  autoScroll  : true,
  layout      : 'form',
  store       : new Ext.data.JsonStore({
    id: 'gridInterfacesAnalog',
    url: 'boards/getGridAnalogInterfacesData',
    root: 'data',
    //,autoLoad: true
    fields: [
      {name: 'id'},
      {name: 'displayName'},
      {name: 'wanpipeInterface'},
      {name: 'nodeId'} //Not displayed
    ]
  }),
  columns    :[
    {header: "ID", width: 35, sortable: true, dataIndex: 'id', css:'padding:5px;'},
    {header: "Name", width: 200, sortable: true, dataIndex: 'displayName', css:'padding:5px;'},
    {header: "WanpipeInterface", width: 100, sortable: true, dataIndex: 'wanpipeInterface', css:'padding:5px;'}
  ],
  viewConfig :{forceFit:true}
}); // eo apply

//gridInterfacesAnalog.addButton("Add Interface",addInterfaceHandler,gridInterfacesAnalog);
//gridInterfacesAnalog.addButton("Remove Interface(s)",deleteInterfaceHandler,gridInterfacesAnalog);

gridInterfacesAnalog.on('rowdblclick', function(grid, rowIndex){
  var record = grid.store.getAt(rowIndex);
  goToNode(record.get('nodeId'));
});

var addInterfaceForm = new Ext.form.FormPanel({
  id        : "add_interface_form",
  region    : "center",
  bodyStyle : 'padding:10px;',
  items     : [{
    name:"interfaceName",
    xtype:"textfield",
    id:"interfaceName",
    fieldLabel:"Interface Name"
  }]
});

var addInterfaceWindow = new Ext.Window({
  title       : 'Add Interface',
  layout      : 'border',
  width       : 505,
  height      : 150,
  closeAction : 'hide',
  plain       : true,
  id          : "add_interface_window"
});

addInterfaceWindow.add( addInterfaceForm );
addInterfaceForm.addButton("Add", addInterface);
addInterfaceForm.addButton("Cancel", formCancel, addInterfaceWindow);

var frmDigitalBoard = new Ext.FormPanel({
  id          : 'frmDigitalBoard',
  title       : 'Boards Details',
  labelWidth  : 75, // label settings here cascade unless overridden
  layout      : 'form',
  frame       : true,
  bodyStyle   : 'padding:5px 5px 0',
  width       : 350,
  labelPad    : 20,
  autoScroll  : true,
  layoutConfig: {
    labelSeparator: ''
  },
  defaults    : {
    width: 230,
    msgTarget: 'side'
  },
  defaultType : 'textfield',
  items       : [{
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  },{
    id: "Paraxip.config.board.digitalBoardID",
    fieldLabel: 'ID',
    name: 'id',
    helpText : "sangomaBoardCommonAttributesID",
    disabled: true
  },{
    id: "Paraxip.config.board.digitalDisplayName",
    fieldLabel: 'Name',
    name: 'displayName',
    helpText : "sangomaBoardCommonAttributesdisplayName"
  },{
    id: "Paraxip.config.board.digitalType",
    fieldLabel: 'Type',
    name: 'type',
    disabled: true
  },{
    id: "Paraxip.config.board.digitalModel",
    fieldLabel: 'Model',
    name: 'model',
    disabled: true
  },{
    id: "Paraxip.config.board.digitalNumInterfaces",
    fieldLabel: 'Number of Interfaces',
    name: 'numInterfaces',
    disabled: true
  }],
  buttons     : [{
    id: "Paraxip.config.board.digitalBoardSaveButton",
    text: 'Save',
    handler: saveBoard
  },{
    id: "Paraxip.config.board.digitalBoardCancelButton",
    text: 'Cancel',
    handler: boardCancelFunc
  }]
});

var panelDigitalBoard = new Ext.Panel({
  id           : 'panelDigitalBoard',
  region       : 'center',
  layout       : 'accordion',
  border       : false,
  closable     : false,
  buttonAlign  : 'right'
});

panelDigitalBoard.add(frmDigitalBoard);
panelDigitalBoard.add(gridInterfacesDigital);

var frmAnalogBoard = new Ext.FormPanel({
  id          : 'frmAnalogBoard',
  title       : 'Boards Details',
  labelWidth  : 75, // label settings here cascade unless overridden
  layout      : 'form',
  frame       : true,
  bodyStyle   : 'padding:5px 5px 0',
  width       : 350,
  labelPad    : 20,
  autoScroll  : true,
  layoutConfig: {
    labelSeparator: ''
  },
  defaults    : {
    width: 230,
    msgTarget: 'side'
  },
  defaultType : 'textfield',
  items       : [{
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  },{
    id: "Paraxip.config.board.analogBoardID",
    fieldLabel: 'ID',
    name: 'id',
    disabled: true
  },{
    id: "Paraxip.config.board.analogDisplayName",
    fieldLabel: 'Name',
    name: 'displayName'
  },{
    id: "Paraxip.config.board.analogType",
    fieldLabel: 'Type',
    name: 'type',
    disabled: true
  },{
    id: "Paraxip.config.board.analogModel",
    fieldLabel: 'Model',
    name: 'model',
    disabled: true
  },{
    id: "Paraxip.config.board.analogNumInterfaces",
    fieldLabel: 'Number of Interface',
    name: 'numInterfaces',
    disabled: true
  },{
    id: "Paraxip.config.board.analogWanpipeNumber",
    xtype: 'numberfield',
    fieldLabel: 'Wanpipe Number',
    name: 'wanpipe'
  }],
  buttons     : [{
    id: "Paraxip.config.board.analogBoardSaveButton",
    text: 'Save',
    handler: saveBoard
  },{
    id: "Paraxip.config.board.analogBoardCancelButton",
    text: 'Cancel',
    handler: boardCancelFunc
  }]
});

var panelAnalogBoard = new Ext.Panel({
  id           : 'panelAnalogBoard',
  region       : 'center',
  layout       : 'accordion',
  border       : false,
  closable     : false,
  buttonAlign  : 'right'
});

panelAnalogBoard.add(frmAnalogBoard);
panelAnalogBoard.add(gridInterfacesAnalog);

// create the Grid
var gridBoards = new Ext.grid.GridPanel({
  id      : "gridBoards",
  layout  : 'fit',
  store   :  new Ext.data.JsonStore({
   id: 'storeBoards',
   url: 'boards/getGridBoardsData',
   root: 'data',
   fields: [
     {name: 'id'},
     {name: 'displayName'},
     {name: 'type'},
     {name: 'model'},
     {name: 'numInterfaces'},
     {name: 'nodeId'} //Not displayed
   ]
  }),
  columns : [
    {header: "ID", width: 35, sortable: true, dataIndex: 'id', css:'padding-top:5px;padding-bottom:5px;'},
    {id:'displayName',header: "Name", width: 100, sortable: true, dataIndex: 'displayName', css:'padding-top:5px;padding-bottom:5px;'},
    {header: "Type", width: 100, sortable: true, dataIndex: 'type', css:'padding-top:5px;padding-bottom:5px;'},
    {header: "Model", width: 100, sortable: true, dataIndex: 'model', css:'padding-top:5px;padding-bottom:5px;'},
    {header: "Number of Interfaces", width: 125, sortable: true, dataIndex:'numInterfaces', css:'padding-top:5px;padding-bottom:5px;'}
  ],  
  stripeRows: true,
  autoExpandColumn: 'displayName',
  title:'Boards Grid',
  viewConfig:{forcefit:true}
});

gridBoards.on('rowdblclick', function(grid, rowIndex){
  var record = grid.store.getAt(rowIndex);
  goToNode(record.get('nodeId'));
});

//Add the buttons to the footer of the grid
// Add and delete button are not ready. The difficulty is in the delete since
// it requires removal of assigned interfaces in the various pannel (physical
// config, isdn config, etc.)
gridBoards.addButton("Add Board",addBoardHandler,gridBoards);
gridBoards.addButton("Remove Board(s)",deleteBoardHandler,gridBoards);

var interfacesNumber = Paraxip.createAnalogInterfaceNumComboBox({id:"Paraxip.config.board.analogInterfaceNumCB"});

var interfaces_num_virtual = Paraxip.createVirtualInterfaceNumField({id:"Paraxip.config.board.virtualInterfaceNum"});

//Selection items
var boardSelection = Paraxip.createBoardSelectionComboBox({id:"Paraxip.config.board.boardSelectionCB"});

boardSelection.on('select', Paraxip.showInterfaceNumberListener.createDelegate(boardSelection,[interfacesNumber, interfaces_num_virtual]));

var addBoardForm = Paraxip.createAddBoardPanel({id:"Paraxip.config.board.addBoardForm"});

var addBoardWindow = new Ext.Window({
  title       : 'Add Board',
  layout      : 'border',
  width       : 505,
  height      : 150,
  closeAction : 'hide',
  plain       : true,
  id          : "add_board_window"
});

addBoardWindow.add( addBoardForm );
addBoardForm.add(boardSelection);
addBoardForm.add(interfacesNumber);
addBoardForm.add(interfaces_num_virtual);
addBoardForm.addButton("Add", addBoard, gridBoards);
addBoardForm.addButton("Cancel", formCancel, addBoardWindow);

addBoardForm.on('afterlayout', Paraxip.showInterfaceNumberListener.createDelegate(boardSelection,[interfacesNumber, interfaces_num_virtual]));

var frmDigitalInterface = new Ext.FormPanel({
  id          : 'frmDigitalInterface',
  title       : 'Interface Details',
  labelWidth  : 75, // label settings here cascade unless overridden
  frame       : true,
  bodyStyle   : 'padding:5px 5px 0',
  width       : 350,
  autoScroll  : true,
  labelPad    : 20,
  layout      : 'form',
  layoutConfig: {
    labelSeparator: ''
  },
  defaults    : {
    width: 230,
    msgTarget: 'side'
  },
  defaultType : 'textfield',

  items       : [{
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  },{
    id: "Paraxip.config.board.digitalInterfaceID",
    fieldLabel: 'ID',
    name: 'id',
    helpText: "sangomaInterfaceCommonAttributesID",
    disabled: true
  },{
    id: "Paraxip.config.board.digitalInterfaceDisplayName",
    fieldLabel: 'Name',
    name: 'displayName',
    helpText: "sangomaInterfaceCommonAttributesdisplayName"
  },{
    id: "Paraxip.config.board.digitalInterfaceWanpipe",
    xtype: 'numberfield',
    fieldLabel: 'Wanpipe',
    name: 'wanpipe',
    helpText: "sangomaDigitalInterfacesinterfacewanpipe"    
  }],

  buttons     : [{
    id: "Paraxip.config.board.digitalInterfaceSaveButton",
    text: 'Save',
    handler: saveInterface
  },{
    id: "Paraxip.config.board.digitalInterfaceCancelButton",
    text: 'Cancel',
    handler: boardCancelFunc
  }]
});

var frmAnalogInterface = new Ext.FormPanel({
  id          : 'frmAnalogInterface',
  title       : 'Interface Details',
  labelWidth  : 75, // label settings here cascade unless overridden
  frame       : true,
  bodyStyle   : 'padding:5px 5px 0',
  width       : 350,
  autoScroll  : true,
  labelPad    : 20,
  layout      : 'form',
  layoutConfig: {
    labelSeparator: ''
  },
  defaults    : {
    width: 230,
    msgTarget: 'side'
  },
  defaultType : 'textfield',

  items       : [{
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  },{
    id: "Paraxip.config.board.analogInterfaceID",
    fieldLabel: 'ID',
    name: 'id',
    helpText : "sangomaInterfaceCommonAttributesID",
    disabled: true
  },{
    id: "Paraxip.config.board.analogInterfaceDisplayName",
    fieldLabel: 'Name',
    name: 'displayName',
    helpText : "sangomaInterfaceCommonAttributesdisplayName"
  },{
    id: "Paraxip.config.board.analogInterfaceWanpipeInterface",
    xtype: 'numberfield',
    fieldLabel: 'Wanpipe Interface',
    name: 'wanpipeInterface',
    helpText:"sangomaAnalogInterfacesinterfacewanpipeInterface"
  }],

  buttons     : [{
    id: "Paraxip.config.board.analogInterfaceSaveButton",
    text: 'Save',
    handler: saveInterface
  },{
    id: "Paraxip.config.board.analogInterfaceCancelButton",
    text: 'Cancel',
    handler: boardCancelFunc
  }]
});

var frmVirtualDigitalBoard = new Ext.FormPanel({
  id          : 'frmVirtualDigitalBoard',
  title       : 'Boards Details',
  labelWidth  : 75, // label settings here cascade unless overridden
  frame       : true,
  bodyStyle   : 'padding:5px 5px 0',
  width       : 350,
  labelPad    : 20,
  autoScroll  : true,
  layout      : 'form',
  layoutConfig: {
    labelSeparator: ''
  },
  defaults    : {
    width: 230,
    msgTarget: 'side'
  },
  defaultType : 'textfield',
  items       : [{
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  },{
    id: "Paraxip.config.board.virtualBoardID",
    fieldLabel: 'ID',
    name: 'id',
    helpText : "sangomaInterfaceCommonAttributesID",
    disabled: true
  },{
    id: "Paraxip.config.board.virtualBoardDisplayName",
    fieldLabel: 'Name',
    name: 'displayName',
    helpText : "sangomaInterfaceCommonAttributesdisplayName"
  },{
    id: "Paraxip.config.board.virtualBoardType",
    fieldLabel: 'Type',
    name: 'type',
    disabled: true
  },{
    id: "Paraxip.config.board.virtualBoardModel",
    fieldLabel: 'Model',
    name: 'model',
    disabled: true
  },{
    id: "Paraxip.config.board.virtualNumInterfaces",
    fieldLabel: 'Number of Interfaces',
    name: 'numInterfaces',
    disabled: true
  }],
  buttons     : [{
    id: "Paraxip.config.board.virtualBoardSaveButton",
    text: 'Save',
    handler: saveBoard
  },{
    id: "Paraxip.config.board.virtualBoardCancelButton",
    text: 'Cancel',
    handler: boardCancelFunc
  }]
});

var panelVirtualDigitalBoard = new Ext.Panel({
  id           : 'panelVirtualDigitalBoard',
  region       : 'center',
  layout       : 'accordion',
  border       : false,
  closable     : false,
  buttonAlign  : 'right'
});

panelVirtualDigitalBoard.add(frmVirtualDigitalBoard);

var frmVirtualDigitalInterface = new Ext.FormPanel({
  id          : 'frmVirtualDigitalInterface',
  title       : 'Virtual Interface Details',
  labelWidth  : 75, // label settings here cascade unless overridden
  frame       : true,
  bodyStyle   : 'padding:5px 5px 0',
  width       : 350,
  autoScroll  : true,
  labelPad    : 20,
  layout      : 'form',
  layoutConfig: {
    labelSeparator: ''
  },
  defaults    : {
    width: 230,
    msgTarget: 'side'
  },
  defaultType : 'textfield',

  items       : [{
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  },{
    id: "Paraxip.config.board.virtualInterfaceID",
    fieldLabel: 'ID',
    name: 'id',
    disabled: true
  },{
    id: "Paraxip.config.board.virtualInterfaceDisplayName",
    fieldLabel: 'Name',
    name: 'displayName'
  },{
    id: "Paraxip.config.board.peerVirtualInterfaceID",
    fieldLabel: 'Peer Virtual Interface ID',
    name: 'peerVirtualInterfaceID',
    helpText: "sangomaVirtualDigitalInterfacesinterfacepeerVirtualInterfaceID",
    xtype: 'numberfield',
    allowNegative: false,
    allowDecimals: false
  }],

  buttons     : [{
    id: "Paraxip.config.board.virtualInterfaceSaveButton",
    text: 'Save',
    handler: saveInterface
  },{
    id: "Paraxip.config.board.virtualInterfaceCancelButton",
    text: 'Cancel',
    handler: boardCancelFunc
  }]
});

